﻿using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Linq;

namespace HeuristicLab.BioBoost.Operators.Mutation {
  [StorableClass]
  public class MultiIntegerVectorManipulator : StochasticMultiBranch<IIntegerVectorManipulator>, IIntegerVectorManipulator, IStochasticOperator {

    public override bool CanChangeName {
      get { return false; }
    }

    protected override bool CreateChildOperation {
      get { return true; }
    }

    public ILookupParameter<IntegerVector> IntegerVectorParameter {
      get { return (ILookupParameter<IntegerVector>)Parameters["IntegerVector"]; }
    }
    public IValueLookupParameter<IntMatrix> BoundsParameter {
      get { return (IValueLookupParameter<IntMatrix>)Parameters["Bounds"]; }
    }

    #region Construction & Cloning
    [StorableConstructor]
    protected MultiIntegerVectorManipulator(bool isDeserializing) : base(isDeserializing) {}
    protected MultiIntegerVectorManipulator(MultiIntegerVectorManipulator orig, Cloner cloner) : base(orig, cloner) {}
    public MultiIntegerVectorManipulator() {
      Parameters.Add(new LookupParameter<IntegerVector>("IntegerVector", "The integer vector that is being manipulating."));
      Parameters.Add(new ValueLookupParameter<IntMatrix>("Bounds", "The lower and upper bounds for each dimension of the vector."));

      foreach (var t in ApplicationManager.Manager.GetTypes(typeof(IIntegerVectorManipulator))) {
        if (t == this.GetType()) continue; // avoid infinite recursion
        IIntegerVectorManipulator instance = null;
        try {
          instance = Activator.CreateInstance(t) as IIntegerVectorManipulator;
        } catch {}
        if (instance != null) Operators.Add(instance);
      }
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiIntegerVectorManipulator(this, cloner);
    }
    #endregion

    protected override void Operators_ItemsReplaced(object sender, CollectionItemsChangedEventArgs<IndexedItem<IIntegerVectorManipulator>> e) {
      base.Operators_ItemsReplaced(sender, e);
      ParameterizeManipulators();
    }

    protected override void Operators_ItemsAdded(object sender, CollectionItemsChangedEventArgs<IndexedItem<IIntegerVectorManipulator>> e) {
      base.Operators_ItemsAdded(sender, e);
      ParameterizeManipulators();
    }

    private void ParameterizeManipulators() {
      foreach (var op in Operators.OfType<IIntegerVectorManipulator>()) {
        op.IntegerVectorParameter.ActualName = IntegerVectorParameter.Name;
      }
      foreach (var op in Operators.OfType<IBoundedIntegerVectorOperator>()) {
        op.BoundsParameter.ActualName = BoundsParameter.Name;
      }
      foreach (var op in Operators.OfType<IStochasticOperator>()) {
        op.RandomParameter.ActualName = RandomParameter.Name;
      }
    }

    public override IOperation InstrumentedApply()
    {
        if (Operators.Count == 0) throw new InvalidOperationException(Name + ": Please add at least one real vector manipulator to choose from.");
        return base.InstrumentedApply();
    }
  }
}
