﻿using HeuristicLab.BioBoost.ProblemDescription;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.BioBoost.Operators.Mutation {
  [StorableClass]
  public class NeighborBasedIntegerVectorMutator : BoundedIntegerVectorManipulator {

    public LookupParameter<BioBoostProblemData> ProblemDataParameter {
      get { return (LookupParameter<BioBoostProblemData>) Parameters["ProblemData"]; }
    }

    public BioBoostProblemData ProblemData {
      get { return ProblemDataParameter.ActualValue; }
    }

    #region Construction & Cloning
    [StorableConstructor]
    protected NeighborBasedIntegerVectorMutator(bool isDeserializing) : base(isDeserializing) {}
    protected NeighborBasedIntegerVectorMutator(NeighborBasedIntegerVectorMutator orig, Cloner cloner) : base(orig, cloner) {}
    public NeighborBasedIntegerVectorMutator() {
      Parameters.Add(new LookupParameter<BioBoostProblemData>("ProblemData", "The problem instance description container."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NeighborBasedIntegerVectorMutator(this, cloner);
    }
    #endregion

    protected override void ManipulateBounded(IRandom random, IntegerVector integerVector, IntMatrix bounds) {
      var source = random.Next(integerVector.Length);
      var target = integerVector[source];
      integerVector[source] = ProblemData.Neighbors.GetRandomNeighbor(target, random, bounds[0, 1]);
    }
  }
}
